#!/bin/sh
#
# /etc/rc.d/rc.S:  System initialization script.
#

### Early userspace

PATH=/sbin:/usr/sbin:/bin:/usr/bin

# Mount proc and sysfs if needed.
mount -at proc,sysfs

# Set the memory manager to panic if memory is exhausted
echo 1 > /proc/sys/vm/panic_on_oom

# STP-670: Force maximum CPU frequency to stop jerky application UI
echo 800000 > /sys/devices/system/cpu/cpu0/cpufreq/scaling_min_freq


### Get the family id to use below for selecting behaviour appropriate for that family.
if [ -r /proc/navico_platform/family_id ]; then
    FAMILYID="$(cut -f 2 /proc/navico_platform/family_id)"
    case "${FAMILYID}" in
     Stingray2) FAMILYID=stingray2 ;;
     Atlantis)  FAMILYID=atlantis ;;
     Cougar)    FAMILYID=cougar ;;
     Condor)    FAMILYID=condor ;; 
     *)         FAMILYID=fixme-familyid ;;
    esac
else
    FAMILYID=fixme-detectmethod
fi

# Squelch output during production.
if [ ! -f /dev/shm/debug ]; then
   exec 1> /dev/null
   exec 2> /dev/null
fi

# Init udev
# Temporary hack for handling Stingray2's ramdisk, which
# populates serial ports in a manner udev doesn't like.
rm -f /dev/ttyS*
if [ -x /etc/rc.d/rc.udev ]; then
  /etc/rc.d/rc.udev start
fi

# Disable complex scheduling for internal solid state storage.
# (leave the CFQ default alone for connectable USB hard disks)
if [ -d /sys/block ]; then
   for BLOCKDEV in \
      $(ls -d /sys/block/mmcblk* /sys/block/mtdblock* /sys/block/ubiblk* /sys/block/loop* 2>/dev/null) ;
   do
      echo noop > ${BLOCKDEV}/queue/scheduler
   done
fi

# Start FUSE, if requested:
if [ -x /etc/rc.d/rc.fuse ]; then
  sh /etc/rc.d/rc.fuse start
fi



### Filesystem maintenance.

# Test to see if the root partition is read-only, like it ought to be.
READWRITE=no
if touch /fsrwtestfile 2>/dev/null; then
  rm -f /fsrwtestfile
  READWRITE=yes
else
  echo "Testing root filesystem status:  read-only filesystem"
fi

# See if a forced filesystem check was requested at shutdown:
if [ -r /etc/forcefsck ]; then
  FORCEFSCK="-f"
fi

# Check the root filesystem:
if [ ! $READWRITE = yes ]; then
  RETVAL=0
  if [ ! -r /etc/fastboot ]; then
    echo "Checking root filesystem:"
    /sbin/fsck $FORCEFSCK -C -a /
    RETVAL=$?
  fi
  if [ $RETVAL -eq 4 ]; then
    # fsck found errors that could not be fixed without human intervention.
    # For our embedded device we need to try really hard to make sure that
    # the system will always boot, so try more thorough checking in 
    # non-interactive mode (assuming that all questions are answered yes)
    /sbin/fsck -f -C -y /
    RETVAL=$?
  fi

  # An error code of 2 or higher will require a reboot.
  if [ $RETVAL -ge 2 ]; then
    # An error code equal to or greater than 4 means that some errors
    # could not be corrected.
    if [ $RETVAL -ge 4 ]; then
      echo
      echo "***********************************************************"
      echo "*** An error occurred during the root filesystem check. ***"
      echo "*** You will now be given a chance to log into the      ***"
      echo "*** system in single-user mode to fix the problem.      ***"
      echo "***                                                     ***"
      echo "*** If you are using the ext2 filesystem, running       ***"
      echo "*** 'e2fsck -v -y <partition>' might help.              ***"
      echo "***********************************************************"
      echo
      echo "Once you exit the single-user shell, the system will reboot."
      echo
      PS1="(Repair filesystem) \#"; export PS1
      sulogin
    else # With an error code of 2 or 3, reboot the machine automatically:
      echo
      echo "***********************************"
      echo "*** The filesystem was changed. ***"
      echo "*** The system will now reboot. ***"
      echo "***********************************"
      echo
    fi
    echo "Unmounting file systems."
    umount -a -r
    mount -n -o remount,ro /
    echo "Rebooting system."
    sleep 2
    reboot -f
  fi
  # Remount the root filesystem in read-write mode
  echo "Remounting root device with read-write enabled."
  mount -w -v -n -o remount /
  if [ $? -gt 0 ] ; then
    echo
    echo "Attempt to remount root device as read-write failed!  This is going to"
    echo "cause serious problems."
    echo
    echo "Please press ENTER to continue, then start looking for the problem. " 
    read junk; 
  fi
else
  echo "Testing root filesystem status:  read-write filesystem"
  echo
  echo "*** ERROR: Root partition has already been mounted read-write. Cannot check!"
  echo
  echo "For filesystem checking to work properly, your system must initially mount"
  echo "the root partition as read only."
  echo
fi # Done checking root filesystem

# If the mtab here is not a link to /proc/mounts, then it is stale.
if [ "$(stat -c '%N' /etc/mtab)" != "'/proc/mounts'" ]; then
   /bin/rm -f /etc/mtab*
   ln -sf /proc/mounts /etc/mtab
fi

# Check all the non-root filesystems:
if [ ! -r /etc/fastboot ]; then
   echo "Checking non-root filesystems:"
   # FIXME:  The real fsck wrapper is broken at the moment.
   # Use this ugly hack instead.
   for BLOCKDEV in $(grep -o "^[^ ]*" /etc/fstab) ; do
      if [ -b "${BLOCKDEV}" ]; then
         FSTYPE="$(grep "${BLOCKDEV}" /etc/fstab | sed 's/^[^ ]* *[^ ]* *//; s/ .*//')"
         case "${FSTYPE}" in
            ext[234])  FSCKFLAGS="-y" ;;
            *) unset FSCKFLAGS ;;
         esac
         if [ -x /sbin/fsck."${FSTYPE}" ]; then
            /sbin/fsck."${FSTYPE}" ${FORCEFSCK} ${FSCKFLAGS} "${BLOCKDEV}"
         fi
      fi
   done
fi

# Check if fstab and mediadaemon configuration is the correct version for this hardware
if ! grep -q "Mass storage" /etc/fstab; then
  cp /etc/fstab /etc/fstab.new

  # Tolerate different driver names: mxsdhci on mx51, sdhci-esdhc-imx on mx61
  if [ -e /sys/devices/platform/mxsdhci.2/mmc_host/mmc*/mmc*:0001 ] || \
     [ -e /sys/devices/platform/sdhci-esdhc-imx.2/mmc_host/mmc*/mmc*:0001 ];
  then
    cat /etc/fstab_emmc >> /etc/fstab.new
    ln -sf /etc/mediadaemon_emmc.conf /etc/mediadaemon.conf
  else
    cat /etc/fstab_nandonly >> /etc/fstab.new
    rm /etc/mediadaemon.conf
  fi

  sync
  mv /etc/fstab.new /etc/fstab
fi

# Mount usbfs:
if grep -wq usbfs /proc/filesystems; then
  if ! grep -wq usbfs /proc/mounts ; then
    if ! grep -wq usbfs /etc/fstab; then
      mount -v usbfs /proc/bus/usb -t usbfs
    else
      mount -v /proc/bus/usb
    fi
  fi
fi

# Mount non-root file systems in fstab, but not NFS or SMB 
# because TCP/IP is not yet configured, and not proc or sysfs
# because those have already been mounted.  Also check that
# devpts is not already mounted before attempting to mount
# it.  With a 2.6.x or newer kernel udev mounts devpts.
echo "Mounting non-root local filesystems:"
if /bin/grep -wq devpts /proc/mounts ; then
  mount -a -v -t nonfs,nosmbfs,nocifs,noproc,nosysfs,nodevpts
else
  mount -a -v -t nonfs,nosmbfs,nocifs,noproc,nosysfs
fi

# If mount failed to attach any of our local file 
# systems then there must be something horribly wrong.
# Run a hard fsck on any block device that isn't mounted.
if [ "$?" -ge 32 ]; then
   for BLOCKDEV in $(cut -f 1 -d '#' /etc/fstab | \
      grep -v swap | \
      awk '{print $1}' ) ;
   do
      if [ -b "${BLOCKDEV}" ]; then
         for MOUNTPT in $(grep "${BLOCKDEV}" /etc/fstab | \
            awk '{print $2}' ) ;
         do
            if ! mountpoint -q "${MOUNTPT}" ; then
              fsck -f -C -v -y "${BLOCKDEV}" > "/dev/shm/fsck.$(basename ${BLOCKDEV})" 2>&1
              mount "${MOUNTPT}"
            fi
         done 
      fi
   done 
fi

# The initial cache mechanism in the initramfs won't find
# shared libraries on /usr.  Cache those now.
if ! grep -q usr /etc/ld.so.cache ; then
  /sbin/ldconfig -C /etc/ld.so.cache.new
  fsync /etc/ld.so.cache.new
  mv /etc/ld.so.cache.new /etc/ld.so.cache
fi


# Workaround for video playback start problems. Cougar, condor and stingray all have video, atlantis not
if [ ${FAMILYID} != "atlantis" ]; then
   # Set the reserved ratio for DMA memory to 1/32 of the normal memory pages
   # (87884 pages, 351536k) so video playback never fails to start.
   #
   # The worst case we need 3 yuyv, 1 rgb16 buffer and 1 2x Size rgb16 buffer
   # = (1.5 * 3 + 2 + 4) * 1024*768 = 8064k = 2016 pages
   # -> reserve 1/32 * 87884 = 2746
   echo 32 > /proc/sys/vm/lowmem_reserve_ratio
fi

# Populate some stuff in /tmp
mkdir -p /tmp/var/run/dbus
mkdir -p /tmp/var/cache/hald
mkdir /tmp/var/lock
mkdir /tmp/var/tmp
mkdir /tmp/oprofile
chmod 0777 /tmp
chmod 0777 /tmp/var/run

# Set system time
if [ -x /etc/rc.d/rc.time ]; then
   /etc/rc.d/rc.time start
fi

# Launch any 'runonce' commands.
for RUNONCE in $(ls /etc/rc.d/runonce.d/* 2>/dev/null) ; do
   if [ ! -f "${RUNONCE}" ]; then
      # Leave any weird stuff alone.
      continue
   fi
   if [ -x "${RUNONCE}" ]; then
      # Clean up after successful completion.
      "${RUNONCE}" && rm -f "${RUNONCE}"
      sync
   fi
done
unset RUNONCE

if grep -i -q CreateCrashLog=true /etc/NOS/Config.ini; then
  touch /dev/shm/enablecrashlog
fi

# If user wants to load defaults OR if home partition has not been generated yet, 
# then delete the contents of the ubi?:home partition
case "${FAMILYID}" in
    stingray2|condor|cougar)
        FACTDEFKEYS="074 078"
        ;;
    atlantis)
        FACTDEFKEYS="074 075"
        ;;
    *)
        FACTDEFKEYS="FIXME"
        ;;
esac

if [ "$(cat /dev/shm/bootkeys)" = "${FACTDEFKEYS}" ] || [ ! -e /home/.fscreated ]; then
   echo "User has requested factory defaults. Deleting home partition..."

   # Work out which UBI volume home is mounted on
   UBIDEV="$(grep /home /proc/mounts | cut -f 1 -d ' ' | cut -f 1 -d ':')"
   UBIDEV="${UBIDEV:-ubi0}"
   for syspath in /sys/class/ubi/${UBIDEV}_?; do
     if [ "$(cat ${syspath}/name)" = "home" ]; then
       vol_num=$(echo ${syspath} | cut -d "_" -f 2)
       VOLDEV="/dev/${UBIDEV}_${vol_num}"
       unset vol_num
       break
     fi
   done
   unset syspath
   unset UBIDEV

   if [ $VOLDEV ]; then
      # Unmount current home fs
      umount /home
      # Truncate ubi volume
      ubiupdatevol ${VOLDEV} -t
      # Create new blank fs, with no space reserved for root
      mkfs.ubifs --reserved=0 ${VOLDEV}
      # Mount new fs
      mount /home

      # Add flag so we remember fs is generated.
      # Need this because FS is auto-generated at mount time if it is blank, the FS defaults to reserving blocks for
      # root (which we don't want), and there is no user space tune fs tool (though it seems that one is possible
      # using some ioctl's, but it has never been implimented)
      touch /home/.fscreated
      sync
   fi
   unset VOLDEV
fi
unset FACTDEFKEYS

# Start loading modules
/etc/rc.d/rc.modules start &

if [ -x /usr/sbin/updatesplashscreen ]; then
  /usr/sbin/updatesplashscreen
fi

### Log system messages when in debug mode
if [ -e /dev/shm/enablecrashlog ] || [ -e /dev/shm/debug ]; then
  [ -x /etc/rc.d/rc.syslog ] && /etc/rc.d/rc.syslog start
  [ -x /etc/rc.d/rc.debuglog ] && /etc/rc.d/rc.debuglog start
fi

### turn on and (optionally) reprogram Sonar and CAN for Stingray2, Condor & Cougar
if [ ${FAMILYID} != "atlantis" ]; then
  if [ -x /etc/rc.d/rc.stm32 ]; then
    /etc/rc.d/rc.stm32 start >/var/run/rc.stm32.log 2>&1 &
  fi
fi

### turn on GPS
if [ -x /etc/rc.d/rc.igps ]; then
  # detection can take up to 5 secs, run in background
  /etc/rc.d/rc.igps start >/var/run/rc.igps.log 2>&1 &
fi

### initialise GPIOs exposed to userspace
if [ -x /etc/rc.d/rc.gpio ]; then
  /etc/rc.d/rc.gpio start
  
  ### set LCD contrast (atlantis 5/7", cougar 7")
  if [ -r /etc/NOS/lcd_contrast ]; then
    /etc/rc.d/rc.gpio lcd_contrast=$(head -n1 /etc/NOS/lcd_contrast)
  fi
fi

### set up the touch screen
case "${FAMILYID}" in
    stingray2|condor|cougar)
        if [ -x /etc/rc.d/rc.touch ]; then
            /etc/rc.d/rc.touch start
        fi
        ;;

    *)  ;;
esac

### bring up network interfaces
if [ -x /etc/rc.d/rc.net ]; then
  /etc/rc.d/rc.net start  

   # If telnet login is required, try to dump the IP address for eth0 out the serial port
   if [ -e /dev/shm/debug ]; then
      unset IPADDR
      (  while [ -z "${IPADDR}" ];
         do
            IPADDR=$(
               ifconfig | \
               grep -A2 eth0 | \
               grep "inet addr:" | \
               sed 's/.*inet addr://; s/ .*//'
            )
            sleep 1
         done
         if [ -n "${IPADDR}" ]; then
            SERIALSPEED=$(stty -F /dev/tts/0 speed)
            stty -F /dev/tts/0 115200
            echo "My IP address for eth0 is ${IPADDR}" > /dev/tts/0
            stty -F /dev/tts/0 "${SERIALSPEED}"
         fi
      ) &
   fi
fi

### Set default key repeat rates (delay and period in ms) for Stingray2, Cougar and Condor
if [ ${FAMILYID} != "atlantis" ]; then
    for kbd in /dev/input/gpiokeys /dev/input/keypad; do
      inpdev --device=$kbd --rptdly 500 --rptprd 125
    done
fi

### run uinput-kbd for Atlantis keyboard.
if [ ${FAMILYID} == "atlantis" ]; then
    if [ -x /etc/rc.d/rc.uinput-kbd ]; then
      /etc/rc.d/rc.uinput-kbd start
    fi
fi

### start D-bus
if [ -x /etc/rc.d/rc.messagebus ]; then
  /etc/rc.d/rc.messagebus start
fi

if [ -e /dev/shm/debug ]; then
  ### run telnetd
  /usr/sbin/telnetd

  ### run sshd
  [ -x /etc/rc.d/rc.sshd ] && /etc/rc.d/rc.sshd start
fi

### start hal
if [ -x /etc/rc.d/rc.hald ]; then
  /etc/rc.d/rc.hald start
fi

# If we've reenabled the serial console,
# move the ttyS0 symlink so NOSApp can't find it (and init will).
if grep -q "console=ttymxc0" /proc/cmdline; then
   mv /dev/ttyS0 /dev/ttymxc0
   ln -sf /dev/null /dev/ttyS0
else
   # Have init open a console prompt on the default VC.
   # (probably the dummy virtual console.)
   ln -sf /dev/null /dev/ttymxc0
   ln -sf /dev/console /dev/tty0
fi

# Start NFS-related services.
if [ -x /etc/rc.d/rc.rpc ]; then
   /etc/rc.d/rc.rpc start
fi
if [ -x /etc/rc.d/rc.nfsd ]; then
   /etc/rc.d/rc.nfsd start
fi

# Enable core-dump handling
[ -x /etc/rc.d/rc.navico-coredump ] && /etc/rc.d/rc.navico-coredump start
# Enable log to media
[ -x /etc/rc.d/rc.navico-logtomedia ] && /etc/rc.d/rc.navico-logtomedia start

# Start mediadaemon. Now done in app script
#if [ -x /etc/rc.d/rc.mediadaemon ]; then
#    /etc/rc.d/rc.mediadaemon start
#fi

# Start screen mirroring service
if [ -x /etc/rc.d/rc.rtspd ]; then
  /etc/rc.d/rc.rtspd start
fi

# Start remote control service
if [ -x /etc/rc.d/rc.remotecontrold ]; then
  /etc/rc.d/rc.remotecontrold start
fi

# Start avahi service
if [ -x /etc/rc.d/rc.avahi ]; then
  /etc/rc.d/rc.avahi start
fi

# Start httpd service
#if [ -x /etc/rc.d/rc.httpd ]; then
#  /etc/rc.d/rc.httpd start
#fi

if [ -x /sbin/watchdog ]; then
  /sbin/watchdog -t 2 -T 15 /dev/watchdog
fi

if [ -x /etc/rc.d/rc.hds-upgrade-compat ]; then
  /etc/rc.d/rc.hds-upgrade-compat start
fi

# Background waiting, so we don't prevent login prompt
(
  /etc/rc.d/rc.modules wait

  # Wait for STM32 micro updates to be complete (stingray2, condor & cougar units)
  if pgrep -f rc.stm32 > /dev/null; then
    echo -n "Waiting for stm32 software upgrade..."
    while pgrep -f rc.stm32 > /dev/null; do
      usleep 25000
    done
    echo Done
  fi

  # Wait for the GPS upgrades to be complete
  if pgrep -f rc.igps > /dev/null; then
    echo -n "Waiting for igps software upgrade..."
    while pgrep -f rc.igps > /dev/null; do
      usleep 25000
    done
    echo Done
  fi

  ### Run the application initialization script
  if [ -x /etc/rc.d/rc.app ]; then
     /etc/rc.d/rc.app start
  fi
) &

