#!/bin/sh

if [ -r /proc/navico_platform/family_id ]; then
  FAMILYID="$(cut -f 2 /proc/navico_platform/family_id)"
else
  FAMILYID=fixme-detectmethod
fi

TARGETUSERNAME=nos
HOMEDIR=/home/${TARGETUSERNAME}
BOOTMODE_FILE=${HOMEDIR}/boot_mode
LAUNCHER=/usr/bin/navico-applauncher
APP=$(readlink -f /usr/bin/NOSApp)
APPLOGFILE=${HOMEDIR}/NOS/.log/NOSApp
TESTAPP=/usr/sbin/TestApp
BURNINCLIENT=/usr/sbin/BurninClient
TS_CAL=/usr/bin/ts_linearize
TIMEOUT=40
APP_PARTITION=/usr
CONFIG_PARTITION=/etc/NOS
MAP_PARTITION=/media/mmcblk0p2
TMP_DIR=/tmp/NOS
RUN_DIR=/var/run
BOOTKEYS=/dev/shm/bootkeys
BURNINFLAG=/tmp/burnin
CAPS="cap_sys_nice,cap_sys_time,cap_sys_tty_config,cap_net_bind_service,cap_net_raw+ep"
NETWORK_CAPS="cap_net_bind_service,cap_net_raw+ep"
DHCP_PROBE_APP=/usr/sbin/dhcp_probe
BUSYBOX_APP=/bin/busybox
OPENTFTP_APP=/usr/bin/opentftpd

APP_PARAMS="-platform navicofb -plugin linuxinputtty:/dev/vc/0 -plugin linuxinputmousehotplug:dir=/dev/input:mask=remotecontrol*"
case "${FAMILYID}" in
  Stingray2|Condor)
    APP_PARAMS="$APP_PARAMS -plugin tslib:/dev/input/touch -plugin navicorotary:/dev/input/rotary"
    ;;
  Cougar)
    APP_PARAMS="$APP_PARAMS -plugin tslib:/dev/input/touch"
    ;;
  Hatchetfish)
    APP_PARAMS="-qws"
    ;;
esac

## The purpose of this section of initialization code is find the
## paths.ini file and read platform values for various settings, using
## reasonable defaults if it fails for any reason.
# TODO: turn this into a couple of calls to a generic function, parsepathsini()
SETTINGSINIPATH="${HOMEDIR}"
RESOURCESPATH=/usr/share/NOS/resources
CHARTDATAPATH=/usr/share/NOS/charts

if [ -r "${HOMEDIR}/paths.ini" ]; then

  # NSW-9100: Ensure paths.ini is up-to-date with /etc/skel
  if [ -r /etc/skel/paths.ini ]; then
    SHA1HOME="$(cd "${HOMEDIR}" && sha1sum paths.ini)"
    SHA1SKEL="$(cd /etc/skel && sha1sum paths.ini)"
    if [ "$SHA1HOME" != "$SHA1SKEL" ]; then
      cp -a /etc/skel/paths.ini "${HOMEDIR}/"
    fi
  fi

  # SettingsIni=./
  if grep -qi ^SettingsIni= "${HOMEDIR}/paths.ini"; then
    SETTINGSINIPATH=$(grep -i ^SettingsIni= "${HOMEDIR}/paths.ini"|cut -d'=' -sf2)
    if [ -n "${SETTINGSINIPATH}" ]; then
      SETTINGSINIPATH="$(cd "${HOMEDIR}" && cd "${SETTINGSINIPATH}" && pwd)"
    fi
  fi
  # Resources=/usr/share/NOS/resources
  if grep -qi ^Resources= "${HOMEDIR}/paths.ini"; then
    RESOURCESPATH=$(grep -i ^Resources= "${HOMEDIR}/paths.ini"|cut -d'=' -sf2)
    if [ -n "${RESOURCESPATH}" ]; then
      RESOURCESPATH="$(cd "${HOMEDIR}" && cd "${RESOURCESPATH}" && pwd)"
    fi
  fi
  # ChartData=/usr/share/NOS/charts/
  if grep -qi ^ChartData= "${HOMEDIR}/paths.ini"; then
    CHARTDATAPATH=$(grep -i ^ChartData= "${HOMEDIR}/paths.ini"|cut -d'=' -sf2)
    if [ -n "${CHARTDATAPATH}" ]; then
      CHARTDATAPATH="$(readlink ${CHARTDATAPATH%/})"
      MAP_PARTITION="${CHARTDATAPATH%/charts*}"
    fi
  fi
fi

if [ -e /etc/navico-coredump.conf ]; then
    . /etc/navico-coredump.conf
fi

deleteflags() {
  rm $TMP_DIR/app-restart
  rm $TMP_DIR/poweroff
  rm $TMP_DIR/app-stop
}

movelogs() {
  if [ -e ${APPLOGFILE} ]; then
    [ -e ${APPLOGFILE}.2 ] && rm ${APPLOGFILE}.2
    [ -e ${APPLOGFILE}.1 ] && mv ${APPLOGFILE}.1 ${APPLOGFILE}.2
    mv ${APPLOGFILE} ${APPLOGFILE}.1
  fi
}

addlogentry() {
  echo $@
  if [ -e /dev/shm/enablecrashlog ]; then
    echo "rc.app: $@" >> ${APPLOGFILE}
    echo "$@" | syslogger -q -n rc.app
  fi
}

enabledebuglogin() {
  # Launch telnetd. It won't launch a second time, because it's bind to the port fails
  /usr/sbin/telnetd

  # Launch ssh
  [ -x /etc/rc.d/rc.sshd ] && /etc/rc.d/rc.sshd start
}

checkmappartition() {
  if [ -e $BURNINFLAG ]; then
    rm $BURNINFLAG
  else
    # Init a new file-system if we either don't have a file system, or the file system is blank
    if ! grep -q $MAP_PARTITION /proc/mounts ; then
      # Not mounted. Need to create file system
      umount $MAP_PARTITION

      if [ "$(grep $MAP_PARTITION /etc/fstab | awk '{print $3}')" = "ext2" ]; then
        local MAP_DEV=$(grep $MAP_PARTITION /etc/fstab | cut -f 1 -d " ")

        mkfs.ext2 -m 0 -L maps $MAP_DEV
        tune2fs -c 0 -i 0 $MAP_DEV
      fi

      mount -o rw $MAP_PARTITION
      mount -o ro,remount $MAP_PARTITION
    fi
  fi
}

init_homedir() {
  mkdir -p $HOMEDIR.new
  cp -a /etc/skel/* $HOMEDIR.new/

  # By default, the root of new fs is owned by root. Change home dir to be owned by correct user
  chown -R ${TARGETUSERNAME}:users $HOMEDIR.new
  sync

  mv $HOMEDIR.new $HOMEDIR
  sync
}

restore_defaults() {
  # "Restore NOS defaults" - create a fresh home dir for $TARGETUSERNAME only
  # A full factory reset recreates the /home partition (refer rc.S)
  mv $HOMEDIR $HOMEDIR.old
  init_homedir
  rm -fr $HOMEDIR.old
  sync
}

read_boot_mode() {
  # Boot mode:
  #   1	TestApp
  #   2	NOSApp / normal boot (default)
  #   3	Burn-in
  BOOTMODE=$(grep -s '^[1-3]$' ${BOOTMODE_FILE} || echo 2)
}

calibratetouchscreen() {
  (
    . /etc/profile

    # Because the touch-screen is a usb device, it can sometimes take a few moments to attach.
    # Wait a reasonable length of time before giving up
    local CNT=0
    while [ ! -e $TSLIB_TSDEVICE ] && [ $CNT -lt 15 ]; do
      sleep 1
      CNT=$(($CNT+1))
    done
    mount -o remount,rw ${CONFIG_PARTITION}
    su -c "${TS_CAL}" -
    sync
    mount -o remount,ro ${CONFIG_PARTITION}
  )
}

launch_nosapp() {
  # Mediadeamon is stupid - its not really a deamon, but actually connects to NOSApp which is
  # acting as the server. So it needs to restart each time nosapp does
  /etc/rc.d/rc.mediadaemon restart

  # Rotate logs on every start
  movelogs


  if [ -e /dev/shm/enablecrashlog ]; then
      su -c "${LAUNCHER} --timeout ${TIMEOUT} \
         --kill \
         --coredumpflag ${NAVICO_COREDUMP_FLAG} \
         --socket ${RUN_DIR}/app_monitor_daemon \
         -- ${APP} ${APP_PARAMS} \
         2>&1 | syslogger -n NOSApp" \
         - ${TARGETUSERNAME} &
      LAUNCHER_PID=$!
  else
      su -c "${LAUNCHER} --timeout ${TIMEOUT} \
         --kill \
         --coredumpflag ${NAVICO_COREDUMP_FLAG} \
         --socket ${RUN_DIR}/app_monitor_daemon \
         -- ${APP} ${APP_PARAMS}" \
         - ${TARGETUSERNAME} &
      LAUNCHER_PID=$!
  fi
  sleep 1

  # Watch the nosapp log, and if it gets too big, rotate it
  ( while true; do
      sleep 60
      if [ -e ${APPLOGFILE} ] && [ "$(du ${APPLOGFILE} | awk '{ print $1 }')" -gt 256 ]; then
        movelogs

        # NOSApp responds to SIGUSR2 by closing and reopening its log file
        kill -USR2 $(pidof NOSApp)
      fi
    done
   ) &
  LOG_WATCH_PID=$!

  DROP_DIR=${TMP_DIR}/features
  # Use inotifyd (busybox) to detect files dropped into $DROP_DIR.  Files that pass
  # the given criteria (refer below) are moved into $CONFIG_PARTITION/features.
  # (Using inotifyd to call echo probably isn't intended behaviour, but it keeps
  # everything in one place.  Refer "inotifyd --help" for full event-flag descriptions).
  INOTIFY_CMD="/sbin/inotifyd /bin/echo ${DROP_DIR}"
  mkdir -p ${DROP_DIR} &&
  chmod a+rw ${DROP_DIR} &&
  ${INOTIFY_CMD} | while read EVENT DIR FILE; do
    [ "${EVENT}" = "w" ] &&		# w event = "Writable file is closed"
    [ "${DIR}" = "${DROP_DIR}" ] &&	# Sanity check (should always be true)
    [ "${FILE:-}" != "" ] &&		# Ignore changes to the directory itself
    [ "${FILE##[0-9]*}" = "" ] &&	# File name only contains digits
    [ ${FILE} -le 255 ] &&		# File name is 0-255
    cd ${DIR} &&
    {
      [ $(/bin/stat -c'%s' ${FILE}) -le 1024 ] && # File size <= 1KiB
      mv ${FILE} ${FILE}.$$ &&		# Make it ours
      sync &&				# Minimise remount-ro time (implied sync, below)
      mount -o remount,rw ${CONFIG_PARTITION} &&
      {
	mkdir -p ${CONFIG_PARTITION}/features &&
	mv ${FILE}.$$ ${CONFIG_PARTITION}/features/${FILE};
	mount -o remount,ro ${CONFIG_PARTITION};
      }
      cd -;
    }
  done >/dev/null 2>&1 &

  if [ "$LAUNCHER_PID" ]; then
    wait $LAUNCHER_PID
  fi

  kill $LOG_WATCH_PID
  pkill -f "${INOTIFY_CMD}"
  unset LOG_WATCH_PID DROP_DIR INOTIFY_CMD

  read_boot_mode

  # check app stop flag
  if [ -f $TMP_DIR/app-stop ]; then
    addlogentry App Stop Requested
    deleteflags
    break

  # check app restart flag
  elif [ -f $TMP_DIR/app-restart ]; then
    addlogentry App Restart
    unset REBOOTING
    STOPCNT=0
    while pgrep NOSApp; do
       sleep 1
       STOPCNT=$(($STOPCNT+1))
       if [ "$STOPCNT" -gt 15 ]; then
           if [ ! -e /dev/shm/debug ]; then
               /sbin/reboot
           fi
           REBOOTING=true
       fi
    done
    unset STOPCNT
    deleteflags

    if [ ! -z "$REBOOTING" ]; then
        break
    fi

  # check poweroff flag
  elif [ -f $TMP_DIR/poweroff ]; then
    addlogentry Power Off
    deleteflags
    /sbin/poweroff
    break

  elif [ "2" != "$BOOTMODE" ]; then
    echo Switching to BOOTMODE: $BOOTMODE

  else
    addlogentry App has died. Rebooting
    if [ ! -e /dev/shm/debug ]; then
      # Give log capture a chance to work
      if [ -e /dev/shm/enablecrashlog ]; then
        sleep 5
      fi
      /sbin/reboot
    fi
    break

  fi
}

showlogo() {
  XDIM=$1
  YDIM=$2
  LOGO=$3

  if [ -n "$XPOS" -a -n "$YPOS" -a -n "$XDIM" -a -n "$YDIM" -a -r "$LOGO" ]; then
    YPOS=$((YPOS-YDIM))
    bootanimator i "$XPOS" "$YPOS" "$LOGO" 2>/dev/null || true
  fi

  unset XDIM YDIM LOGO
}

start() {
  #TODO remove below this before Condor RTM1 vvvvvvvvvvvvvvvvvvvvv
  if [ "${FAMILYID}" = 'Condor' ] && grep -q 'ProductId=NSS' $CONFIG_PARTITION/Config.ini; then
    mount -o remount,rw $CONFIG_PARTITION
    rm $CONFIG_PARTITION/Config.ini
    sync
    mount -o remount,ro $CONFIG_PARTITION
  fi
  #TODO remove above this before Condor RTM1 ^^^^^^^^^^^^^^^^^^^^^

  # Make a default config.ini if one does not exist
  if [ ! -s $CONFIG_PARTITION/Config.ini ]; then

    mount -o remount,rw $CONFIG_PARTITION

    case "${FAMILYID}" in
      Stingray2)
    cat > $CONFIG_PARTITION/Config.ini <<EOF
[General]
INIVersion=2.1

[Product]
ProductId=NSS
ProductRegion=AMER
ProductDataset=InsightUS

[ApplicationDefaults]
DefaultChartProvider=Navico
DefaultUnits=ImperialCoastal
DefaultVideoFormat=NTSC
DefaultTimeZone=-06:00
DefaultDateFormat=MM/dd/yyyy
DefaultTimeFormat=12hr
EOF
        ;;
      Condor)
    cat > $CONFIG_PARTITION/Config.ini <<EOF
[General]
INIVersion=2.1

[Product]
ProductId=Condor
ProductRegion=AMER
ProductDataset=

[ApplicationDefaults]
DefaultChartProvider=Navico
DefaultUnits=Imperial
DefaultVideoFormat=NTSC
DefaultTimeZone=-06:00
DefaultDateFormat=MM/dd/yyyy
DefaultTimeFormat=12hr
EOF
        ;;
      Atlantis)
    cat > $CONFIG_PARTITION/Config.ini <<EOF
[General]
INIVersion=2.1

[Product]
ProductId=HDSGen2
ProductRegion=AMER
ProductDataset=InsightUS

[ApplicationDefaults]
DefaultChartProvider=Navico
DefaultUnits=Imperial
DefaultVideoFormat=N/A
DefaultTimeZone=-06:00
DefaultDateFormat=MM/dd/yyyy
DefaultTimeFormat=12hr
EOF
        ;;
      Cougar)
    cat > $CONFIG_PARTITION/Config.ini <<EOF
[General]
INIVersion=2.1

[Product]
ProductId=HDSGen2Touch
ProductRegion=AMER
ProductDataset=InsightUS

[ApplicationDefaults]
DefaultChartProvider=Navico
DefaultUnits=Imperial
DefaultVideoFormat=N/A
DefaultTimeZone=-06:00
DefaultDateFormat=MM/dd/yyyy
DefaultTimeFormat=12hr
EOF
        ;;
    esac

    sync
    mount -o remount,ro $CONFIG_PARTITION
  #TODO remove below this before Cougar RTM1 vvvvvvvvvvvvvvvvvvvvv
  else # config.ini file has contents
    # check to see if we need to convert a pre-release device
    if grep -q 'ProductId=Cougar' $CONFIG_PARTITION/Config.ini; then
      mount -o remount,rw $CONFIG_PARTITION
      sed -i '/ProductId=/ s/Cougar/HDSGen2Touch/' $CONFIG_PARTITION/Config.ini
      sync
      mount -o remount,ro $CONFIG_PARTITION
    fi
  #TODO remove above this before Cougar RTM1 ^^^^^^^^^^^^^^^^^^^^^
  fi

  if [[ "$(getcap $APP | cut -d " " -f 3)" != "$CAPS" ]] || [[ "$(getcap $LAUNCHER | cut -d " " -f 3)" != "$CAPS" ]]; then
    echo "Setting application capabilities..."
    mount -o remount,rw $APP_PARTITION
    setcap $CAPS $APP
    setcap $CAPS $LAUNCHER
    sync
    mount -o remount,ro $APP_PARTITION
  fi

  if [ -x $DHCP_PROBE_APP ]; then
    if [[ "$(getcap $DHCP_PROBE_APP | cut -d " " -f 3)" != "$NETWORK_CAPS" ]]; then
      mount -o remount,rw $APP_PARTITION
      setcap $NETWORK_CAPS $DHCP_PROBE_APP
      sync                             
      mount -o remount,ro $APP_PARTITION
    fi
  fi

  if [ -x $BUSYBOX_APP ]; then
    if [[ "$(getcap $BUSYBOX_APP | cut -d " " -f 3)" != "$NETWORK_CAPS" ]]; then
      setcap $NETWORK_CAPS $BUSYBOX_APP
      sync
    fi
  fi
  
  if [ -x $OPENTFTP_APP ]; then
    if [[ "$(getcap $OPENTFTP_APP | cut -d " " -f 3)" != "$NETWORK_CAPS" ]]; then
      mount -o remount,rw $APP_PARTITION
      setcap $NETWORK_CAPS $OPENTFTP_APP
      sync
      mount -o remount,ro $APP_PARTITION
    fi
  fi

  # Make sure the temp directory actually exists
  if [ ! -d $TMP_DIR ]; then
    mkdir -p $TMP_DIR
  fi
  chown ${TARGETUSERNAME}:users ${TMP_DIR}

  # Make sure the home directory actually exists
  if [ ! -e $HOMEDIR ]; then
    init_homedir
  fi

  XRES=$(sed -n '/[[:digit:]]x/{s/^[^[:digit:]]*\([[:digit:]]\+\)x.*$/\1/;p}' /sys/class/graphics/fb0/mode)
  YRES=$(sed -n '/x[[:digit:]]/{s/^.*x\([[:digit:]]\+\).*$/\1/;p}' /sys/class/graphics/fb0/mode)
  if [ -n "$XRES" -a -n "$YRES" ]; then
    XPOS=28 # left offset
    if [ "${FAMILYID}" = 'Condor' ]; then
      YPOS=94 # top offset + logo height = 28 + 66
    else
      YPOS=$((YRES-28)) # bottom offset
    fi

    if [ -r "$CONFIG_PARTITION/Config.ini" ]; then
      PRODUCT_DATASET=$(grep -i '^ProductDataset=' "$CONFIG_PARTITION/Config.ini"|tr -d '\r'|cut -d= -f2)
    fi

    if [ -r "$SETTINGSINIPATH/Settings.ini" ] && \
      grep -qi "^WeatherEnabled=true" "$SETTINGSINIPATH/Settings.ini"
    then
      case "$PRODUCT_DATASET" in
      Polaris)
        showlogo 125 66 "$RESOURCESPATH/weather/sirius_weather_logo2.bmp"
        ;;
      *)
        showlogo 125 66 "$RESOURCESPATH/weather/sirius_weather_logo.bmp"
        ;;
      esac
    fi

    case "$PRODUCT_DATASET" in
    InsightUS|LakeInsight|NauticInsight)
      showlogo 125 66 "$RESOURCESPATH/insight_logo.bmp"
      ;;
    esac

    unset XPOS YPOS PRODUCT_DATASET
  fi

  # if you want to capture a screenshot (RGB565) of the splash screen then uncomment the following line
  #dd if=/dev/fb0 of=/tmp/fb0-${XRES}x$YRES bs=$((XRES*YRES*2)) count=1 2>/dev/null || true
  unset XRES YRES

  # clean up any symlinks and torrent files on the chart partition
  mount -o rw,remount /media/factorydata
  find /media/factorydata -maxdepth 1 -type l -exec rm {} \;
  rm /media/factorydata/*.torrent
  mount -o ro,remount /media/factorydata

  cd "$HOMEDIR"

  ( while true; do
    read_boot_mode

    if [ -r ${BOOTKEYS} ]; then
      case "${FAMILYID}" in
        Stingray2|Condor)
          case "$(cat ${BOOTKEYS})" in
            "025")	# PAGES key - TestApp
              BOOTMODE=1
              ;;
            "050")	# MENU key - Touch screen calibration
              calibratetouchscreen
              BOOTMODE=2
              ;;
          esac
          ;;

        Atlantis)
          case "$(cat ${BOOTKEYS})" in
            108|"105 108"|"106 108") # down | left+down | right+down - TestApp
              BOOTMODE=1
              ;;
            025) # PAGES key - soft reset (delete Settings.ini)
              rm "${SETTINGSINIPATH}/Settings.ini"
              ;;
          esac
          ;;

        Cougar)
          case "$(cat ${BOOTKEYS})" in
            "074") # Pg Down key - TestApp
              BOOTMODE=1
              ;;
            "025")  # PAGES key - soft reset (delete Settings.ini)
              rm "${SETTINGSINIPATH}/Settings.ini"
              ;;
            "045")  # WPT key - Touch screen calibration
              calibratetouchscreen
              BOOTMODE=2
              ;;
          esac
          ;;
      esac

      mv ${BOOTKEYS} ${BOOTKEYS}.old 
    fi

    case ${BOOTMODE} in
      1)
        # TestApp
        # Currently there are some requirements that must be met - refer http://nzwiki.marine.net.int/wiki/StingrayII/TestApp
        enabledebuglogin

        mount -o remount,rw ${CONFIG_PARTITION}

        # Stop mediadaemon, and ensure all the auto-mounted SD cards are un-mounted
        /etc/rc.d/rc.mediadaemon stop
        for MOUNTPOINT in /media/mmcblk*; do
            umount $MOUNTPOINT
        done
        unset MOUNTPOINT

        # FIXME Temporary workaround for STP-184
        for b in 500; do
          echo $b >/sys/class/backlight/pwm-backlight/brightness
        done

        # Still run as root, but wrap inside su to set environment correctly.
        # Pass the location of paths.ini to TestApp so it has a chance to be
        # smarter about where to find things
        if [ -r ${HOMEDIR}/paths.ini ]; then
          su -c "${TESTAPP} ${APP_PARAMS} ${HOMEDIR}/paths.ini" -
        else
          su -c "${TESTAPP} ${APP_PARAMS}" -
        fi
        EXITCODE=$?

        sync
        mount -o remount,ro ${CONFIG_PARTITION}

        # For TestApp exit codes, refer TestApp/Widgets/tSpecialModes.h
        case ${EXITCODE} in
          0|16)
            echo "Normal mode"
            restore_defaults # and implied: rm -f ${BOOTMODE_FILE}; sync
            ;;
          17)
            echo "Demo mode"
            # <TODO> ??!
            ;;
          18)
            echo "Burnin mode"
            echo 3 >${BOOTMODE_FILE}
            touch $BURNINFLAG
            sync
            ;;
          19)
            echo "TestMode"
            echo 1 >${BOOTMODE_FILE}
            sync
            ;;
          20)
            echo "Reboot"
            /sbin/reboot
            exit
            ;;
          143)
            echo "App Stop Requested"
            exit
            ;;
           *)
            # Signals are 128+SIGNUM (eg 143 for SIGTERM)
            echo "$0: Unhandled exit code ${EXITCODE}"
            break
            ;;
        esac
        ;;

      2)
        # NOSApp / normal boot
        launch_nosapp
        ;;

      3)
        # Burn-in
        enabledebuglogin

        checkmappartition

        mount -o remount,rw ${MAP_PARTITION}

        # BurninClient needs an IP address to function properly.  Spin.
        echo "BurninClient: Waiting for an IP address to appear on eth0."
        unset IPADDR
        while [ -z "${IPADDR}" ];
        do
          IPADDR=$(
             ifconfig | \
             grep -A1 '^eth0[^:]' | \
             grep "inet addr:" | \
             sed 's/.*inet addr://; s/ .*//'
          )
          sleep 1
        done
        unset IPADDR

        if grep -q "^00:0e:c2" /sys/class/net/eth0/address ||
           grep -q "^00:0e:91" /sys/class/net/eth0/address; then
          BURNINSERVERS="169.254.1.1:169.254.2.2:169.254.3.3:169.254.4.4:169.254.5.5"
          PRODUCT_ID="$(grep 'ProductId=' ${CONFIG_PARTITION}/Config.ini | tr -d '\r' | cut -d= -f2)"
          PRODUCT_REGION="$(grep 'ProductRegion=' ${CONFIG_PARTITION}/Config.ini | tr -d '\r' | cut -d= -f2)"
          FINAL_ASSY_PART_NUM="$(grep 'FinalAssyPartNum=' ${CONFIG_PARTITION}/Config.ini | tr -d '\r' | cut -d= -f2)"
          PRODUCT_DATASET="$(grep '^ProductDataset=' $CONFIG_PARTITION/Config.ini | tr -d '\r' | cut -d= -f2)"
          BURNIN_DATASET="${PRODUCT_ID}${PRODUCT_REGION}${PRODUCT_DATASET}"
          if [ -n "${FINAL_ASSY_PART_NUM}" ]; then
            TITLE="--title ${FINAL_ASSY_PART_NUM}"
          else
            FINAL_ASSY_PART_NUM="000-0000-00"
          fi
          /etc/rc.d/rc.syslog stop
          /etc/rc.d/rc.syslog start-remote
          # Still run as root, but wrap inside su to set environment correctly.
          # Don't use touch input plugins so that BurninClient cannot be killed by pressing on the screen
          su -c "${BURNINCLIENT} ${APP_PARAMS} ${TITLE} \
                                 --dataset ${BURNIN_DATASET} \
                                 --serverlist ${BURNINSERVERS} \
                                 --partnum ${FINAL_ASSY_PART_NUM} \
                                 --dest /media/factorydata" - &
        else
          su -c "${BURNINCLIENT} ${APP_PARAMS} --title Error --message \"Invalid MAC address\" --color red" - &
        fi

        [ -e /dev/ttyS0 ] && nmealistener /dev/ttyS0 4800 ${BURNINCLIENT}
        EXITCODE=$?

        sync
        mount -o remount,ro ${MAP_PARTITION}

        case ${EXITCODE} in
          0|16)
            echo "Normal mode"
            restore_defaults # and implied: rm -f ${BOOTMODE_FILE}; sync
            ;;
          17)
            echo "Demo mode"
            # <TODO> ??!
            ;;
          18)
            echo "Burnin mode"
            # Already in burnin mode, restart BurninClient
            ;;
          19)
            echo "TestMode"
            echo 1 >${BOOTMODE_FILE}
            sync
            ;;
          20)
            echo "Reboot"
            /sbin/reboot
            exit
            ;;
          143)
            echo "App Stop Requested"
            exit
            ;;
           *)
            # Signals are 128+SIGNUM (eg 143 for SIGTERM)
            echo "$0: Unhandled exit code ${EXITCODE}"
            break
            ;;
        esac
        ;;
    esac
    done
  ) &
}

stop() {
  # Force script to exit when NOSApp killed
  # (signals are lost through applauncher)
  touch $TMP_DIR/app-stop

  IFS=,
  KILL_LIST="${APP},${TESTAPP},${BURNINCLIENT}"
  for app in ${KILL_LIST};
  do
    if ! pgrep ${app} > /dev/null 2>&1; then
      echo "${app} not running"
    else
      pkill ${app}
      while pgrep ${app} > /dev/null 2>&1; do
        sleep 0.1
      done
    fi
  done

  NMEA_PID=$(pgrep -f '/bin/sh /usr/sbin/nmealistener')
  if [ -n "${NMEA_PID}" ]; then
    kill -STOP ${NMEA_PID}
    pkill -P ${NMEA_PID}
    kill ${NMEA_PID}
    kill -CONT ${NMEA_PID}
  fi
}

case "$1" in
    start)
        start
        ;;
    stop)
        stop
        ;;
    restart)
        stop
        start
        ;;
    *)
        echo "Usage: $0 {start | stop | restart}"
        ;;
esac
