# A set of common helper functions usefull for iMX51 updaters

### Immediate execution

UPDDIR="$(dirname "${UPDFILE}")"
THISCODENAME="$(grep CODENAME /etc/updater/updater.conf | cut -d= -f2)"
if grep -q "i.MX 6" /proc/cpuinfo; then
  ISMX6=true
else
  ISMX6=false
fi

### Functions

# Find the framebuffer geometry of fb0.
# The y-axis may be double that of standard sizes.
display_res() {
  local FBGEOM="$(cat /sys/class/graphics/fb0/virtual_size 2>/dev/null)"
  local DISPLAYRES=unknown

  case "${FBGEOM}" in
    480,*)   DISPLAYRES=480x480 ;;
    640,*)   DISPLAYRES=640x480 ;;
    800,*)   DISPLAYRES=800x600 ;;
    1024,*)  DISPLAYRES=1024x768 ;;
    "")
      echo "unittype:  WARNING:  Framebuffer geometry is blank." 1>&2
      echo "unittype:  Assigning 480x480 as fallback." 1>&2
      DISPLAYRES=480x480
      ;;
    *)
      echo "unittype:  WARNING:  Framebuffer geometry is an unexpected value." 1>&2
      echo "unittype:  FBGEOM case faulted on value='${FBGEOM}'" 1>&2
      echo "unittype:  Assigning 480x480 as fallback." 1>&2
      DISPLAYRES=480x480
      ;;
  esac

  echo $DISPLAYRES
}

# 4GB and 8GB MLC has 512k ERASESIZE and 4k PAGESIZE
# 1GB NAND has 256k ERASESIZE and 4k PAGESIZE
# 128MB NAND has 128k ERASESIZE, 2k PAGESIZE and 512b sub-page size
nand_geometry() {
  local ERASESIZE=$(($(cat /sys/class/mtd/mtd0/erasesize) / 1024))
  local PAGESIZE=$(($(cat /sys/class/mtd/mtd0/writesize) / 1024))

  echo ${PAGESIZE}k${ERASESIZE}k
}

nand_page_size() {
  cat /sys/class/mtd/mtd0/writesize
}

nand_erase_size() {
  cat /sys/class/mtd/mtd0/erasesize
}

## Try and figure out the NAND size that is present
nand_size() {
  local SIZES=$(cat /proc/mtd | grep mtd | cut -d " " -f 2)
  local NANDSIZE=0

  # Sum the MTD partitions. This is a bit flawed, because there is no rule that mtd partitions
  # must use the whole device, or that the mtd partitions cannot overlap. Our mx51 platform
  # uses the whole device in non-overlapping partitions, so this will work.
  for size in $SIZES; do
    size=$((0x$size))
    NANDSIZE=$(($NANDSIZE + $size))
  done
  unset size

  echo $NANDSIZE
}

have_emmc() {
  # Tolerate different driver names: mxsdhci on mx51, sdhci-esdhc-imx on mx61
  if [ -e /sys/devices/platform/mxsdhci.2/mmc_host/mmc*/mmc*:0001 ] || \
     [ -e /sys/devices/platform/sdhci-esdhc-imx.2/mmc_host/mmc*/mmc*:0001 ];
  then
    return 0
  else
    return 1
  fi
}

ram_size() {
  # For Atlantis, there are two possible RAM SKUs: 512MB and 128MB.
  # The 128MB cards are built with R278 populated, meaning HW_ID.1 will read low.
  # Do not rely on /proc/meminfo (and friends) to reveal the actual installed RAM size
  #	as it is possible to run the 128MB bootloader on a 512MB card
  if [ 0 -eq $(( `cat /proc/navico_platform/cpu_card_ver` & 2 )) ]; then
    echo 128
  else
    echo 512
  fi
}

show_debug_msg() {
  ${DEBUG} && {
     echo "Showing 'debug mode' notification."
     splashy_update "print Just in case you didn't already
know, you are running this device
in debug mode!"
     sleep 6
     splashy_update "print This updater will be more verbose
than normal and there are other
differences in system behaviour
that you may not be expecting."
     sleep 9
     splashy_update "print Don't say I didn't warn you!"
     sleep 3
     splashy_update "clear"
  }
}

# Get the MTD used for UBI
find_ubi_mtd_partition() {
  local UBIMTD="$(grep -i '"system"' /proc/mtd | sed 's/^mtd//; s/:.*//')"
  if [ -z "${UBIMTD}" ]; then
    return 1
  fi
  echo $UBIMTD
  return 0
}

attach_check_ubi() {
  local NUMUBIPARTS
  local UBIMTD=$(find_ubi_mtd_partition)
  if [ -z $UBIMTD ]; then
    return 1
  fi

  echo UBIMTD = $UBIMTD 1>&2

  ubiattach -m $UBIMTD || true

  if have_emmc; then
    NUMUBIPARTS=3
  else
    NUMUBIPARTS=4
  fi

  if [ "$(cat /sys/class/ubi/ubi0/volumes_count)" != $NUMUBIPARTS ]; then
    return 1
  fi

  return 0
}

start_splashy() {
  # If updater packages a version of splashy, use that. Older mx51 builds include splashy in the init ramdisk, 
  # so if this is not present it is not necesarily fatal
  if sha1check splashy.tar.xz; then
    tar -xOf "${UPDFILE}" splashy.tar.xz | tar x -C /
    [ -x /etc/rc.d/rc.splashysetup ] && /etc/rc.d/rc.splashysetup
    if [ -x /usr/sbin/splashy -a -x /usr/sbin/splashy_update ]; then
      USE_SPLASHY=true
    fi
  elif [ -x /usr/sbin/splashy_theme -a -x /usr/sbin/splashy -a -x /usr/sbin/splashy_update ]; then
    USE_SPLASHY=true
  fi

  # Make splashy_update failure nonfatal.
  splashy_update () {
     [ -n "$USE_SPLASHY" ] && command "splashy_update" "$@" 2>/dev/null || true;
     return 0;
  }

  [ -n "$USE_SPLASHY" ] && (
    cd /etc/splashy
    if [ -e "config-logoprogress.xml" ]; then
      ln -sf "config-logoprogress.xml" config.xml
    fi
  )
  [ -n "$USE_SPLASHY" ] && splashy boot || true;

  PROGRESS=0
  splashy_update "progress ${PROGRESS}"
}

start_progress_bar() {
  # Background progress loop.
  (
    set +x
    BREAKPOINT=50
    while [ ! -f /stop-progress-flag.$$ ]; do
      if [ ${PROGRESS} -ge ${BREAKPOINT} ]; then
        BREAKPOINT=$((${BREAKPOINT}+(100-${BREAKPOINT})/2))
        PROGRESSBARDELAY=$((2*${PROGRESSBARDELAY}))
        if [ ${PROGRESSBARDELAY} -ge 42 ]; then
          PROGRESSBARDELAY=42
        fi
      fi
      PROGRESS=$((${PROGRESS}+1))
      if [ ${PROGRESS} -gt 99 ]; then
        PROGRESS=99
      fi
      splashy_update "progress ${PROGRESS}"
      sleep ${PROGRESSBARDELAY}
    done
  ) &
}

stop_progress_bar() {
  touch /stop-progress-flag.$$
  splashy_update "clear"
  splashy_update "progress 100"
}

update_csm_atlantis() {
  # Find the first .csu file.
  local CSUFILE="$( \
    grep -A 42 -m 1 '^###SHA1MAGIC###$' "${0}" | \
    grep -i '.csu$' | \
    sed 's/[^ ]\{40\}  //' | \
    sort | \
    uniq | \
    head -n 1
  )"

  if [ -n "${CSUFILE}" -a \
       "${THISCODENAME}" = atlantis -a \
       -c /dev/sonar ];
  then
    if sha1check "${CSUFILE}" ; then
      tar xf "${UPDFILE}" "${CSUFILE}"

      mount -w /mnt
      if have_emmc; then
         mount -w /mnt/usr  
      fi
      mount -t tmpfs tmpfs /mnt/tmp
      mount -t proc proc /mnt/proc
      mount -o bind /dev /mnt/dev
      cp -a "${CSUFILE}" /mnt/tmp/

      SH_FLAGS="$-"
      set +e
      chroot /mnt sh -l -c "CSMUpdater -h" >/dev/null 2>/dev/null
      local RETVAL=$?
      set -"${SH_FLAGS}"
      unset SH_FLAGS

      case "${RETVAL}" in
      0) # command accepts -h, it's the commandline edition.
         ${DEBUG} && splashy_update "print Programming CSM"

         if chroot /mnt sh -l -c "CSMUpdater -h" 2>/dev/null | grep -q -- --splashy ; then
            # This tool can output splashy percentage data.  Hook it to the pipe.

            CSMProgressPipe() {
               # $1 = start percentage
               # $2 = end percentage

               STARTPCT="$((${1:-0}))"
               PCTRANGE="$(($((${2:-100}))-${STARTPCT}))"

               while read STDIN; do
                  # CSM stdout text passthrough
                  echo "${STDIN}"

                  PCT="$(echo "${STDIN}" | grep -o '[0-9]*')"
                  if [ -n "${PCT}" ]; then
                     PCT="$((${PCT}*${PCTRANGE}/100))"
                     echo "splashy_update \"progress $((${PCT}+${STARTPCT}))\""
                  fi
               done

               unset STARTPCT
               unset PCTRANGE
               unset STDIN
               unset PCT
            }

            touch /stop-progress-flag.$$
            chroot /mnt sh -l -c "CSMUpdater \"/tmp/${CSUFILE}\"" | CSMProgressPipe
            rm /stop-progress-flag.$$

            unset -f CSMProgressPipe

         else
            # This tool can't print splashy percentage output.
            chroot /mnt sh -l -c "CSMUpdater \"/tmp/${CSUFILE}\""
         fi
         ;;

      1) # Received a simple error code.  Must the be QWS edition.
         # The graphical CSM updater requires Qt's graphics stack to be complete,
         # which in turn requires /dev/gsl_kmod to be present.
         # (This gpu.ko module might not match the running kernel.  Risk it.)
         chroot /mnt sh -l -c "modprobe gpu" || break
         sleep 1

         touch /stop-progress-flag.$$
         splashy_update "exit"

         chroot /mnt sh -l -c "CSMUpdater -qws \"/tmp/${CSUFILE}\""

         # We're not really shutting down,
         # but this gives us a 100% progress bar immediately on launch
         splashy shutdown
         ;;

      *) # No CSMUpdater installed at all.  Fault.
         ;;
      esac

      umount /mnt/dev 
      umount /mnt/proc
      umount /mnt/tmp
      if have_emmc; then
         umount /mnt/usr
      fi
      umount /mnt

    else
      ${DEBUG} && {
         splashy_update "print Bad SHA1 on CSM firmware."
         sleep 5
      }
    fi
  fi
}

# Check a component exists and it is not corrupted
# $1 = component name
# $2 = fail if component missing
validate_component() {
  # What are we upgrading?
  echo "Calculating work for ${1}..."
  ${DEBUG} && {
      splashy_update "clear"
      splashy_update "print Calculating work for
${1}..."
   }

   sha1check ${1}
}

corrupt_component_fail() {
  touch /stop-progress-flag.$$
  if "${DEBUG}" ; then
    echo "ERROR:  Bad update for ${1}.  Halted!"
    splashy_update "print ERROR:  Bad update for
${1}
Refusing to program bad software.
Please replace your update file."
  else
     echo "ERROR:  This update is damaged.  Halted!"
     splashy_update "print ERROR:  This update is damaged.
Refusing to program bad software.
Please replace your update file."
  fi
  set +x
  while true; do 
     splashy_update "noop"
     sleep 42
  done
}

write_mtd_partition_from_file() {
  local partition_name=$1
  local filename=$2

  local PARTITION_SIZE_HEX=$(grep -i "${partition_name}" /proc/mtd | \
         sed 's/^mtd.\: *//; s/ .*$//')

  ${DEBUG} && echo -n ", length 0x${PARTITION_SIZE_HEX}"

  local PARTITION_DEV=$(grep -i "\"${partition_name}\"" /proc/mtd | \
         sed 's/: .*$//' )
  ${DEBUG} && echo ", device ${PARTITION_DEV}"

  # Make sure we're not flashing an image larger than our partition.
  local PARTITION_SIZE_DEC=$(printf "%d" "0x${PARTITION_SIZE_HEX}")
  local COMPONENT_SIZE_DEC=$(stat -c %s "$filename")

  ${DEBUG} && echo "DEBUG: Partition size is ${PARTITION_SIZE_DEC} bytes"
  ${DEBUG} && echo "DEBUG: Source image is   ${COMPONENT_SIZE_DEC} bytes"
  if [ "${COMPONENT_SIZE_DEC}" -gt "${PARTITION_SIZE_DEC}" ]; then
    echo "ERROR: $filename too big for target"
    splashy_update "print ERROR: $filename is
too big for target"
         sleep 30
         splashy_update "exit"
         exit 1
  fi

  echo "Erasing flash: ${partition_name}"
  ${DEBUG} && splashy_update "print Erasing flash:
${partition_name}"

  flash_eraseall -q /dev/"${PARTITION_DEV}"

  ## Write region
  # Using nandwrite
  #   nandwrite doesn't do streaming input, extract to a temporary file.
  #   WARNING:   This temporary file must be less than (system memory - running kernel)
  echo "Writing new image: $filename"
  ${DEBUG} && splashy_update "print Writing new image:
$filename"

  nandwrite -p /dev/${PARTITION_DEV} $filename
}

write_mtd_partition() {
  local partition_name=$1
  local filename=$2

  tar -tf "${UPDFILE}" $filename || corrupt_component_fail $filename

  write_mtd_partition_from_file $partition_name $filename

}

update_kernel_or_fail() {
  local filename=$1

  tar xf "${UPDFILE}" $filename
  write_mtd_partition kernel $filename
  rm $filename
}

update_rescue_or_fail() {
  local filename=$1

  tar xf "${UPDFILE}" $filename
  write_mtd_partition rescue $filename
  rm $filename
}

update_check_bi_swap_enabled() {
  # Don't rely on the /sys/devices/platform/mxc_nandv2_flash.0/disable_bi_swap file if we are in
  # a factory update (since the running kernel will always start with bi_swap ON)
  local BI_SWAP
  if egrep -iqs unsupervised-update /proc/cmdline ; then
    local SAVE_BI_SWAP=$(cat /sys/devices/platform/mxc_nandv2_flash.0/disable_bi_swap)

    echo 1 > /sys/devices/platform/mxc_nandv2_flash.0/disable_bi_swap
    dd if=/dev/mtd0 | strings > ubootstrings
    echo $SAVE_BI_SWAP > /sys/devices/platform/mxc_nandv2_flash.0/disable_bi_swap

    if grep -q u-boot ubootstrings; then
      echo u-boot found
      BI_SWAP=$(cat ubootstrings | grep "bi_swap=[01]" | head -n 1)
      echo BI_SWAP = $BI_SWAP

      rm ubootstrings
  
      if [ -z "$BI_SWAP" ]; then
        # If bi_swap can't be found in u-boot environment, assume the installed bootloader will be pre- bi_swap change
        return 1
      elif [ "$BI_SWAP" = "bi_swap=1" ]; then
        return 0
      else
        return 1
      fi
    else
      # No u-boot detected. Flash is (probably) blank. Turn on bi_swap
      echo u-boot not found

      rm ubootstrings
      return 0
    fi
  else
    # For normal in-field updates, the kernel could be very old, in which case the disable_bi_swap
    # file will not exist. In this case the bi_swap is always OFF

    if [ ! -e /sys/devices/platform/mxc_nandv2_flash.0/disable_bi_swap ]; then
      return 1
    elif  [ "$(cat /sys/devices/platform/mxc_nandv2_flash.0/disable_bi_swap)" = "1" ]; then
      return 1
    else
      return 0
    fi
  fi
}

update_boot_imx51() {
  local filename=$1

  mkdir boottmp
  tar xOf "${UPDFILE}" $filename | xzcat | tar x -C boottmp
  
  if update_check_bi_swap_enabled; then
    BI_SWAP=1
  else
    BI_SWAP=0
  fi
  echo Setting bi_swap=$BI_SWAP
  sed -i "s/bi_swap=[01]/bi_swap=${BI_SWAP}/" boottmp/all.env

  # If not forcing the CODENAME then preserve any existing one
  if [ -z "$CONVCODENAME" ]; then
    CONVCODENAME=$(sed -n 's/.*navico_sku\.codename=\([^ ]*\).*/\1/p' /proc/cmdline)
  fi

  # If we 'are' Flensburg then flag it
  if [ "${CONVCODENAME}" = "flensburg" ]; then
    sed -i 's/navico_sku=${navico_sku}/& navico_sku.codename=flensburg/' boottmp/all.env
  fi

  ENVPAD=boottmp/envpad-arm

  ENV_OFFSET=$( ${ENVPAD} --info | sed 's/.*offset=\([0-9]*\).*/\1/' )
  ${ENVPAD} --info

  local BOOT_LOADER
  local BL_VARIANT
  if [ $(ram_size) = 128 ]; then
    BL_VARIANT="-128ram"
  else
    BL_VARIANT=""
  fi
  BOOT_LOADER=boottmp/boot${BL_VARIANT}.bin


  for copy in 1 2; do
    dd if=${BOOT_LOADER} bs=${ENV_OFFSET} count=1
    cat boottmp/all.env | ${ENVPAD} -
  done > uboot.bin
  rm -fR boottmp

  write_mtd_partition_from_file boot uboot.bin

  rm uboot.bin

  return 0
}

update_boot_imx6() {
  local filename=$1

  tar xOf "${UPDFILE}" kobs-ng.gz | gzip -c -d > kobs-ng
  chmod +x kobs-ng
  tar xOf "${UPDFILE}" $filename > boot.bin

  # Need the flash_eraseall, because kobs-ng won't overwrite the area with the 
  # environment in it (and we want to reset to default environment)
  flash_eraseall /dev/mtd0

  ./kobs-ng init -w -v boot.bin

  rm kobs-ng
  rm boot.bin

  return 0
}

update_boot_or_fail() {
  local filename=$1
  local MIN_VER=$2

  if [ "${MIN_VER}" != "-1" ]; then
    CURRENT_VER=$(cat /proc/navico_platform/boot_ver)
    echo "Bootloader is version ${CURRENT_VER}."
    ${DEBUG} && splashy_update "print Bootloader is version
${CURRENT_VER}."
    if [ ${CURRENT_VER} -eq 0 ]; then
       echo "It is not safe to update this version, skipping."
       ${DEBUG} && {
         splashy_update "print It is not safe to update this
version, skipping."
         sleep 1
       }
       unset CURRENT_VER
       return 0
    fi

    # Is the current version of the bootloader less than the minimum allowed bootloader?
    if [ ${MIN_VER} -gt ${CURRENT_VER} ];
    then
      echo "Bootloader is out of date and needs updating."
      ${DEBUG} && splashy_update "print Bootloader is out of date and
needs updating."
    else
      echo "Bootloader is already up-to-date."
      ${DEBUG} && {
        splashy_update "print Bootloader is already up-to-date."
        sleep 1
      }
      unset MIN_VER CURRENT_VER
      return 0
    fi
    unset MIN_VER CURRENT_VER
  fi

  
  if $ISMX6; then
    update_boot_imx6 $filename
  else
    update_boot_imx51 $filename
  fi
}

# Finds the ubi in format of eg ubi0_1
find_ubi_volume_num_from_name() {
  volume_name=$1

  # Scan the names of the ubi volumes, to check if this component is a volume image
  # Relies on UBI volumes not getting the same name as mtd partitions
  local UBIVOL=
  for syspath in /sys/class/ubi/ubi0_?; do
    if [ "$(cat ${syspath}/name)" = "${volume_name}" ]; then
      vol_num=$(echo ${syspath} | cut -d "_" -f 2)
      UBIVOL="ubi0_${vol_num}"
      unset vol_num
      break
    fi
  done
  unset syspath
  echo $UBIVOL
}

# If the updater is stored on an MMC device, then wait
# for the partition to disappear (card removal).  For all other
# sources, wait for the power button to be pressed.
wait_for_media_reboot() {

  # Get the device number (in decimal) of updater
  MEDIA_DEV=$(stat -c '%d' "${UPDFILE}")
  # Generate a regex to use with /proc/partitions
  MEDIA_RE="^[[:space:]]*$(($MEDIA_DEV/256))[[:space:]]\+"	# major
  MEDIA_RE="${MEDIA_RE}$(($MEDIA_DEV%256))[[:space:]]\+"	# minor
  MEDIA_RE="${MEDIA_RE}[[:digit:]]\+[[:space:]]\+mmcblk"	# blocks, name

  set +x  # squelch annoying infinite loop.

  # Omit -q(uiet) flag for first grep
  if grep "${MEDIA_RE}" /proc/partitions; then
     splashy_update "print Upgrade completed.
Remove media to reboot."

     while grep -q "${MEDIA_RE}" /proc/partitions; do
       splashy_update "noop"
     done
  else
    splashy_update "print Upgrade completed.
Press the power key to reboot."
     
    case "${THISCODENAME}" in
      stingray2|flensburg)
        while [ "$(keytest status)" != "016" ]; do
          splashy_update "noop"
        done
        ;;
      atlantis)
        while [ "$(testkey power)" != 1 ]; do
          splashy_update noop
        done
        ;;
    esac
  fi
  splashy_update "exit"
  echo 'reboot -f' > ~/.updater/bootaction
}



libupdate_resolvelinkpath() {
   # Find a 'real' subdirectory by resolving symlinks.
   # Unlike readlink's canonicalize options, this is designed to operate
   # on a mountpoint where absolute links are relative to the top of the filesystem
   # (ie, an inactive rootfs mounted beneath an alternate live rootfs).
   # $1= initial target to resolve.
   # $2= base directory to prepend per iteration and to the final output.  "The chroot".
   # On success, return 0 and print a string containing the absolute resolved location.
   # On failure, return 1.

   local TARGET
   local NEWTARGET
   local BASEPATH
   local LINKCWD

   TARGET="${1}"
   BASEPATH="${2:+"${2}/"}"

   while [ -h "${TARGET}" ]; do
      NEWTARGET="$(readlink "${TARGET}")"
      LINKCWD="$(cd "$(dirname "${TARGET}")" && pwd)/"

      # Was this symlink relative or absolute?
      if echo "${NEWTARGET}" | grep -q "^/" ; then
         # Absolute.  Prepend the basepath.
         NEWTARGET="${BASEPATH}${NEWTARGET}"
      else
         # Relative.  Resolve against the link's parent directory and prepend the basepath.
         NEWTARGET="${BASEPATH}${LINKCWD}${NEWTARGET}"
      fi

      # Don't get stuck.
      if [ "${TARGET}" = "${NEWTARGET}" ]; then
         echo "Symlink loop.  Failing." >&2
         return 1
      fi

      TARGET="${NEWTARGET}"
   done

   echo "${TARGET}"
   return 0
}


libupdate_findparentmountpoint() {
   # Given a directory name, traverse the tree towards / until a mountpoint is found.
   # $1= starting directory
   # On success, return 0
   # On failure, return 1

   local DIR
   DIR="${1}"

   # This is a real directory, right?
   if ! (cd "${DIR}"); then
      echo "ERROR:  Can't chdir into ${1}" >&2
      return 1
   fi

   # Climb toward the root of the tree.
   while [ "${DIR}" != "/" -a -n "${DIR}" ] && \
      ! mountpoint -q "${DIR}" ;
   do
      DIR="$(echo "${DIR}" | rev | cut -f 2- -d '/' | rev)"
   done
   echo "${DIR:-/}"
   return 0
}



libupdate_sims_delete() {
   # Remove all files in the simulator directory.
   # Requires CODENAME to be set.
   # On success, returns 0
   # On failure, returns 1

   local SIMSTARGET
   local SIMSMNTPT

   # Only valid for these products.  Seed SIMSTARGET.
   case "${THISCODENAME}" in
   atlantis)
      SIMSTARGET="/mnt/usr/share/NOS/simulator" ;;
   cougar)
      SIMSTARGET="/mnt/usr/share/NOS/simulator" ;;
   *)
      echo "Not valid for ${THISCODENAME} products." >&2
      return 1
      ;;
   esac
   SIMSTARGET="$(libupdate_resolvelinkpath "${SIMSTARGET}" /mnt)"

   # If the directory doesn't exist, don't try.
   if [ -z "${SIMSTARGET}" ]; then
      echo "ERROR: Empty simulator directory parameter."  >&2
      return 1
   fi
   if ! (cd "${SIMSTARGET}"); then
      echo "ERROR: Simulator directory '${SIMSTARGET}' doesn't exist." >&2
      return 1
   fi

   SIMSMNTPT="$(libupdate_findparentmountpoint "${SIMSTARGET}")"

   (  cd "${SIMSTARGET}" || exit 1
      mount -o remount,rw "${SIMSMNTPT}" || {
         echo "ERROR:  Couldn't remount ${SIMSMNTPT} as read-write." >&2
         exit 1
      }
      rm -rf * 
      mount -o remount,ro "${SIMSMNTPT}"
   )
   return $?
}



libupdate_sims_copyfromcard() {
   # Copy in the "sims/" directory from the update media.
   # Requires CODENAME and UPDDIR to be set.
   # Status messages will be printed on stdout.

   # Only valid for these products.  Seed SIMSTARGET.
   case "${THISCODENAME}" in
   atlantis)
      SIMSTARGET="/mnt/usr/share/NOS/simulator" ;;
   cougar)
      SIMSTARGET="/mnt/usr/share/NOS/simulator" ;;
   *)
      echo "Not valid for ${THISCODENAME} products." >&2
      return 1
      ;;
   esac

   local SIMSSRC
   local DIR
   local SIMSTARGET
   local SIMSMNTPT

   SIMSTARGET="$(libupdate_resolvelinkpath "${SIMSTARGET}" /mnt)"

   if [ -z "${SIMSTARGET}" ]; then
      echo "ERROR:  Empty simulator installation target string." >&2
      return 1
   fi
   if ! (cd "${SIMSTARGET}"); then
      echo "ERROR:  Can't chdir into ${SIMSTARGET}" >&2
   fi

   SIMSMNTPT="$(libupdate_findparentmountpoint "${SIMSTARGET}")"
   mount -o remount,rw "${SIMSMNTPT}" || {
      echo "ERROR:  Couldn't remount ${SIMSMNTPT} as read-write." >&2
      return 1
   }


   SIMSSRC="$(ls -d "${UPDDIR}"/[Ss][Ii][Mm][Ss] 2>/dev/null | head -n 1)"
   if [ -n "${SIMSSRC}" ]; then
      splashy_update "print Copying in your simulators"
      echo "Copying in your simulators"

      # Use a fixed list of directory names.
      # FATfs is case insensitive, so the end user will probably not
      # match the case sensitive path we expect.
      # Luckily, this cuts the other way.
      # We can chdir into non-matching paths and get the files we want.
      # Note:  If your card uses a 'real' filesystem, it will need to match.
      (  cd "${SIMSSRC}"
         for DIR in \
            Demo/Scripts/ \
            Demo/Images/ \
            Logs/AIS/ \
            Logs/Radar/ \
            Logs/Weather/ \
            Logs/Sonar/ \
            ;
         do (
            cd "${DIR}" 2>/dev/null || exit 0
            mkdir -p "${SIMSTARGET}/${DIR}"
            export SIMSTARGET
            export DIR
            find . -maxdepth 1 -type f -exec sh -c 'cat "{}" > "${SIMSTARGET}/${DIR}/{}"' \;
         ); done
      )
      sync

      # If there are any loose files directly in SIMSSRC, file them away appropriately.
      # It's easier to do this with a temporary script called as an argument to find -exec
      cat << _EOF > /tmp/process_sim.sh
#!/bin/sh
set -x
# arg 1 = path to simulator directory on the target
# arg 2 = filename
case "\${2}" in
*.[Ss][Ll][Gg]|*.[Ss][Ll]2)
   mkdir -p "\${1}/Logs/Sonar/"
   cat "\${2}" > "\${1}/Logs/Sonar/\${2}"
   ;;
*.[Ww][Xx])
   mkdir -p "\${1}/Logs/Weather/"
   cat "\${2}" > "\${1}/Logs/Weather/\${2}"
   ;;
*.[Rr][Ss][Ff])
   # All .rsf files can be considered radar logs because
   # there is no facility to create AIS logs.
   # If a new AIS log needs to be loaded, then it needs  
   # to be placed in the fixed directory structure above.
   mkdir -p "\${2}/Logs/Radar/"
   cat "\${2}" > "\${1}/Logs/Radar/\${2}"
   ;;
*.[Pp][Nn][Gg])
   mkdir -p "\${1}/Demo/Images/"
   cat "\${2}" > "\${1}/Demo/Images/\${2}"
   ;;
*.[Tt][Xx][Tt])
   if grep -q "^DEMO SCRIPT FILE\$" "\${2}" ; then
      mkdir -p "\${1}/Demo/Scripts/"
      cat "\${2}" > "\${1}/Demo/Scripts/\${2}"
   fi
   ;;
*.[Ss][Mm][Ff])
   # TODO.  Requires R&D agreement on target location, 
   # as well as implementation within the simulator.
   true
   ;;
esac
_EOF
      chmod +x /tmp/process_sim.sh
      (  cd "${SIMSSRC}"
         export SIMSTARGET
         find . -maxdepth 1 -type f -exec sh -c '/tmp/process_sim.sh "${SIMSTARGET}" "{}"' \;
      )
      rm -f /tmp/process_sim.sh

      mount -o remount,ro "${SIMSMNTPT}"
   fi

   return $?
}

