#!/bin/sh
# Update script

# System handshake magic
UNAME=armv7l
CODENAME=condor
CODENAMELIST=condor
DATESTAMP=20150924101010

# This progress bar delay is just used as an approximate speed for moving the progress bar
# After the bar has passed 75%, it will slow to 1/2 this rate, and 1/4 this rate after 90%
PROGRESSBARDELAY=1

# Print out the updater version. This needed by the factory to validate the correct version is being
# programmed when doing automated CPU card programming
# NOTE: makeupd.sh will modify this line
echo UPDATEVERSION=VesselView7-5.0-43.579-32766-r1

# TODO: Detect need for UBI repair automatically. This should only ever need to be done once 
#       at the factory. Probably use logic of if (!ubiattach) || (volumecount != 3) { ubirepair }


DEBUG=${DEBUG:-false}
if [ -e /dev/shm/debug ]; then
   DEBUG=true
fi
${DEBUG} && set -x

# If anything goes awry, exit early.
set -e

# We expect to be invoked from the /tmp directory.
# We expect to have working /proc and /sys directories, as well as a populated /dev

### This upgrade script expects the following environment variables to be present:
# UPDFILE -- a path to the '.upd' update file.
#    ex:  UPDFILE = /path/to/foo.upd
#    This file path is found during the discovery phase of the initramfs
#    and will be relative to the ramfs root.
if [ -z "${UPDFILE}" ]; then
   echo "Internal error: Bad environment.  UPDFILE was not set."
   exit 1;
fi


### This file also encapsulates SHA1 sums of the componets it expects to operate upon.
# These need to be checked before the script proceeds.
# TODO: Use hardware-accelerated SHA1
# TODO: Test the system sha1sum for tampering.
SHA1CMD=sha1sum
find_component_sha1() {
   if [ -z "${1}" ]; then
      echo "no file to check, aborting"
      return 1;
   fi

   local THISSHA1="$(
      grep -A 500 "^###SHA1MAGIC###$" "${0}" | \
         grep -o "[^ ]\{40\}  ${1}$"
   )"
   if [ "$?" -ne "0" ] || [ -z $THISSHA1 ]; then
      echo "File \"${1}\" not found in internal hash table." >&2
      return 1
   fi
   THISSHA1="$(echo "${THISSHA1}" | cut -f 1 -d ' ' )"

   echo $THISSHA1
   return 0
}

sha1check () {
   if [ -z "${1}" ]; then
      echo "no file to check, aborting"
      return 1;
   fi

   local EXTERNAL=0
   if [ "${1}" = "-external" ]; then
      EXTERNAL=1
      shift 1
   fi

   local THISSHA1=$(find_component_sha1 ${1})
   ${DEBUG} && echo "Stored SHA1 for ${1} is ${THISSHA1}"

   local LOCALSHA1

   if [ -z $THISSHA1 ]; then
      return 1
   fi

   if [ "${EXTERNAL}" = 1 ]; then
      if [ ! -r "$(dirname "${UPDFILE}")/${1}" ]; then
         echo "Couldn't open external file ${1}"
         return 1
      fi
      LOCALSHA1="$("${SHA1CMD}" "$(dirname "${UPDFILE}")/${1}" | cut -f 1 -d ' ')"
   else
      LOCALSHA1="$(tar -xOf ${UPDFILE} ${1} | "${SHA1CMD}" | cut -f 1 -d ' ' )"
   fi

   ${DEBUG} && echo "Calculated SHA1 for ${1} is ${LOCALSHA1}"

   if [ "${THISSHA1}" != "${LOCALSHA1}" ]; then
      echo "ERROR: Update corrupt.  File \"${1}\" failed checksum."
      echo "Please delete this update and replace it with a new copy."
      return 1;
   else
      ${DEBUG} && echo "DEBUG: successful sha1sum check: ${1}"
   fi
   return 0;
}


sha1check libupdate.sh || {
  echo "fail"
}

tar -xf "${UPDFILE}" libupdate.sh

. ./libupdate.sh

start_splashy

#show_debug_msg

splashy_update "print Preparing update..."

DISPLAYRES=$(display_res)
NANDGEOMETRY=$(nand_geometry)
NANDSIZE=$(nand_size)

echo $DISPLAYRES $NANDGEOMETRY $NANDSIZE
have_emmc && echo eMMC Detected



###  Prepare early tools.
# None for now

echo "Showing 'validating update' notification."
splashy_update "print Validating update. Please wait...
Do not turn off during update, as
this may damage your device."

start_progress_bar

if find_component_sha1 simulator.tar.gz; then
  HAVE_SIM_PAYLOAD=true
  validate_component simulator.tar.gz || corrupt_component_fail simulator
else
  HAVE_SIM_PAYLOAD=false
fi

if find_component_sha1 root-overlay.tar.xz; then
  HAVE_ROOTOVERLAY_PAYLOAD=true
  validate_component root-overlay.tar.xz || corrupt_component_fail root-overlay
else
  HAVE_ROOTOVERLAY_PAYLOAD=false
fi

validate_component root.tar.xz || corrupt_component_fail root
validate_component usr.tar.xz || corrupt_component_fail usr
validate_component boot.bin || corrupt_component_fail boot
if $ISMX6; then
  validate_component kobs-ng.gz || corrupt_component_fail boot
fi
validate_component kernel.bin || corrupt_component_fail kernel

# If the UBI seems to be missing or corrupt, just bail out
attach_check_ubi || {
   echo "ERROR: UBI volumes corrupted. Halted"
   splashy_update "print ERROR:  System is damaged.
Return for service."

   while true; do 
      splashy_update "noop"
      sleep 42
   done
}

ROOTUBI=$(find_ubi_volume_num_from_name root)
if [ -z $ROOTUBI ]; then
  corrupt_component_fail "ubi root"
fi

echo "Showing 'installing update' notification."
splashy_update "print Installing update. Please wait...
Do not turn off during update, as
this may damage your device."

update_kernel_or_fail kernel.bin

MINBOOTVER=3
update_boot_or_fail boot.bin $MINBOOTVER

${DEBUG} && splashy_update "print Updating root partition..."
ubiupdatevol -t /dev/${ROOTUBI}
mount -o rw /mnt
tar -xOf "${UPDFILE}" root.tar.xz | xzcat | tar -x -C /mnt

${DEBUG} && splashy_update "print Updating usr partition..."

if have_emmc; then
  mkfs.ext2 -m 0 -L usr /dev/mmcblk0p1
  if sha1check tune2fs-1.41.12-static.xz; then
    tar -xOf "${UPDFILE}" tune2fs-1.41.12-static.xz | xzcat > ./tune2fs
    chmod 0755 ./tune2fs
    ./tune2fs -c 0 -i 0 /dev/mmcblk0p1
    rm -f ./tune2fs
  fi

  mount -o rw /mnt/usr
fi

tar -xOf "${UPDFILE}" usr.tar.xz | xzcat | tar -x -C /mnt/usr

if $HAVE_ROOTOVERLAY_PAYLOAD; then
  echo "Writing root overlay files"
  ${DEBUG} && splashy_update "print Installing root overlay..."
  tar -xOf "${UPDFILE}" root-overlay.tar.xz | xzcat | tar -x -C /mnt
fi

mount --bind /dev /mnt/dev
mount --bind /sys /mnt/sys
mount --bind /proc /mnt/proc
for TTY in /dev/ttymxc*; do
  ln -s ${TTY#/dev/} ${TTY/mxc/S}
done

# Create the initial ld.so.cache for our new rootfs.
chroot /mnt sh -l -c "ldconfig"

# Update firmware during update, each boot will check
if [ -x /mnt/etc/rc.d/rc.stm32 ]; then
  ${DEBUG} && splashy_update "print Checking CAN firmware..."
  chroot /mnt /etc/rc.d/rc.stm32 deepcheck || true
fi
if [ -x /mnt/etc/rc.d/rc.igps ]; then
  ${DEBUG} && splashy_update "print Checking GPS firmware..."
  chroot /mnt /etc/rc.d/rc.igps check || true
fi

for TTY in /dev/ttyS*; do
  rm $TTY
done
umount /mnt/proc
umount /mnt/sys
umount /mnt/dev

${DEBUG} && splashy_update "print Checking for simulator files..."
HAVE_SIM_MEDIA=false
if ls -d "${UPDDIR}"/[Ss][Ii][Mm][Ss] >/dev/null 2>/dev/null ; then
   HAVE_SIM_MEDIA=true
fi

if ${HAVE_SIM_MEDIA}; then
   # Clear all existing simulators.
   if ! mountpoint -q /mnt/media/factorydata; then
      mount /mnt/media/factorydata
   fi
   libupdate_sims_delete
fi

if $HAVE_SIM_PAYLOAD; then
   echo "Writing simulator files"
   ${DEBUG} && splashy_update "print Writing simulator files"

   if ! mountpoint -q /mnt/media/factorydata; then
      mount /mnt/media/factorydata
   fi
   mount -o remount,rw /mnt/media/factorydata

   DESTPREFIX=/mnt/media/factorydata/simulator
   cd ${DESTPREFIX}
   EXISTINGFILES=$(find -type f | sed 's/\.\///')
   cd /

   tar xOf "${UPDFILE}" simulator.tar.gz | tar xz manifest
   for f in ${EXISTINGFILES}; do
     echo -n "${f}: "
     if ! grep -q $f manifest; then
       echo "Not required. Deleting."
       rm ${DESTPREFIX}/${f}
     else
       NEWSHA=$(grep $f manifest | awk '{print $1}')
       EXISTINGSHA=$(sha1sum ${DESTPREFIX}/${f} 2>/dev/null | awk '{print $1}')
       if [ "${NEWSHA}" != "${EXISTINGSHA}" ]; then
         echo "Updated. Adding to copy list."
         COPYFILES="${COPYFILES} $(echo ${f})"
       else
         echo "Unchanged. Leaving existing file."
       fi
     fi
   done

   for f in $(awk '{print $2}' manifest); do
     if [ ! -e ${DESTPREFIX}/${f} ]; then
       echo "${f}: New. Adding to copy list."
       COPYFILES="${COPYFILES} $(echo ${f})"
     fi
   done

   if [ "${COPYFILES}" != "" ]; then
      echo Copying new files...
      ( cd $DESTPREFIX; tar xOf "${UPDFILE}" simulator.tar.gz | tar xvz ${COPYFILES}; )
   fi

   sync
   unset COPYFILES DESTPREFIX EXISTINGFILES NEWSHA EXISTINGSHA f
fi

if ${HAVE_SIM_MEDIA}; then
   # Copy in the card's "sims/" directory
   libupdate_sims_copyfromcard
fi

if mountpoint -q /mnt/media/factorydata ; then
   umount /mnt/media/factorydata
fi

# Create the initial ld.so.cache for our new rootfs.
chroot /mnt sh -l -c "ldconfig"
sync


if have_emmc; then
  umount /mnt/usr
fi

umount /mnt

sync

update_csm_atlantis
stop_progress_bar

splashy_update "clear"
splashy_update "progress 100"

echo "Update Finished!"

wait_for_media_reboot

# Done!
exit 0
exit 0
###SHA1MAGIC###
9330dd4df8c68e3bf7b7a18b9cad95a6e008a176  libupdate.sh
a81f5fb3033a93d27979a3d63eb87d3570653978  kernel.bin
aad7cc07f50fd872c6abda251752cb368d4c55f2  boot.bin
463f1558ac43374f3b3211986a018769ac331ade  usr.tar.xz
0b6d9f689eae8b30376440513aec3f2619dbb41e  root.tar.xz
1ea4e80e56542264f406af9b375fc2184c861db3  root-overlay.tar.xz
